/**************************************************************************************************
 *
 * Classname:	Pathfinder	
 * Properties:	Contains the GUI and handles user interactive events
 * Superclass:	javax.swing.JFrame
 *
 **************************************************************************************************/

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import java.io.*; //look and feel

public class Pathfinder extends JFrame
{
	private JMenuBar menu;
	private JMenu file, difficulty, help;
	private JMenuItem file_new_game, file_exit, help_about;
	private JPanel panel;			//This panel will contain the actual minefield
	private Container contentPane;		//The content pane, where all graphic components will be placed
	private JButton reset;			//A reset button, restarts the game
	private Menu_Handler menu_actions;	//The action-handler responsible for dealing with menu actions
	private Board field;			//This is the internal representation of the minefield
	private int x_size = 40;		//The x-size of the minefield (Default setting: 15)
	private int y_size = 40;		//The y-size of the minefield (Default setting: 15)
	private int mines = 200;			//The number of mines (Default setting: 30)
	private boolean game_running = false;	//False when the game has not started, true when the user has clicked the first time
	
	private Misc Misc;
	private List closed;
	private Walllayer Walllayer;
	
	
	/*
	 * Methodname:	Pathfinder
	 * Purpose:	The default constructor of the main class, this is where the flow of the program is controlled
	 * Parameters:	None
	 * Returns:	An object of type Pathfinder
	 */
	public Pathfinder()
	{
		//First-time initiations
		contentPane = getContentPane();
		menu_actions = new Menu_Handler();
		panel = new JPanel();
		reset = new JButton("Reset (F2)");
		
		field = new Board(x_size, y_size, mines);			//Instanciate the minefield (internal representation)
		field.init_board();									//Initiate the minefield
		
		panel.addMouseListener(new Mouse_Handler());		//This MouseAdapter handles mouse events on the minefield
		reset.addActionListener(new Button_Handler());		//This ActionListener handles events generated by the reset-button
		
		//Some settings for the main frame
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);		//The application exits when the user closes the window
		setLocation(50,50);								//The default location of the window
		setResizable(false);								//Window not resizable
		setTitle("Pathfinder");				
		setIconImage(new ImageIcon("sys_icon.gif").getImage());	//Set the app-icon to our beautiful 32x32 pixel mine
		
		//Instanciate the menu
		menu = new JMenuBar();
		file = new JMenu("File");
		help = new JMenu("Help");
		file_new_game = new JMenuItem("New game");
		file_exit = new JMenuItem("Exit");
		help_about = new JMenuItem("About");
		
		//Set mnemonics and accelerators for the menu
		file.setMnemonic('F');
		help.setMnemonic('H');
		file_new_game.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F2, 0));
		file_exit.setAccelerator(KeyStroke.getKeyStroke('X', InputEvent.CTRL_MASK));
		
		//Build the menubar
		file.add(file_new_game);
		file.addSeparator();
		file.add(file_exit);		
		help.add(help_about);
		menu.add(file);
		menu.add(help);
		
		//Add actionlisteners for the menu buttons
		file_new_game.addActionListener(menu_actions);
		file_exit.addActionListener(menu_actions);
		help_about.addActionListener(menu_actions);
		
		setJMenuBar(menu);
		
		//Construct the main panel: The maze 
		contentPane.setLayout(new BorderLayout());
		panel.setLayout(new GridLayout(field.get_y_size(), field.get_x_size()));
		init_graphic_minefield();							//Each call to this method redraws the maze
		
		contentPane.add(panel, BorderLayout.CENTER);
		contentPane.add(reset, BorderLayout.SOUTH);
		
		pack();				//Sets the window size to dimensions that are sums of all the components preferred sizes
		setVisible(true);
	}
	
	/*
	 * Methodname:	main()
	 * Purpose:	Starts the game by creating an instance of the Pathfinder class
	 * Parameters:	args (An array of command-line arguments)
	 * Returns:	None
	 */
	public static void main(String[] args)  throws IOException 
	{
		JFrame.setDefaultLookAndFeelDecorated(true);
		// set the "lookandfeel", you can choose different types
		try { UIManager.setLookAndFeel("javax.swing.plaf.metal.MetalLookAndFeel"); }
		catch (Exception e) { }
		
		new Pathfinder();
	}
	
	/*
	 * Methodname:	init_graphic_minefield()
	 * Purpose:	Adds the labels of the internally represented squares (Board.minefield) to the graphic representation
	 * Parameters:	None
	 * Returns:	None
	 */
	private void init_graphic_minefield()
	{
		for (int n = 0; n < field.get_y_size(); n++)
		{
			for (int m = 0; m < field.get_x_size(); m++)
			{
				panel.add(field.get_maze()[m][n].get_label());
			}
		}
	}
	
	/*
	 * Methodname:	reset_game()
	 * Purpose:	Contains actions to be performed when the user restarts the game
	 * Parameters:	None
	 * Returns:	None
	 */
	private void reset_game(int new_x_size, int new_y_size, int new_num_mines)
	{	
		game_running = false;
		
		if (panel.getMouseListeners().length == 0)
			panel.addMouseListener(new Mouse_Handler());		
		
		contentPane.setVisible(false);
		panel.setVisible(false);
		contentPane.remove(panel);	//Remove the main panel from the frame
		panel.removeAll();			//Remove all contents (squares) from the main panel
		
		field = new Board(new_x_size, new_y_size, new_num_mines);	//Create a new minefield
		field.init_board();		
		
		panel.setLayout(new GridLayout(field.get_y_size(), field.get_x_size()));		
		
		init_graphic_minefield();
		contentPane.add(panel, BorderLayout.CENTER);
		contentPane.setVisible(true);
		panel.setVisible(true);		
		pack();
		
	}
	
	/**************************************************************************************************
	 * Classname:	Mouse_Handler 
	 * Properties:	Handles mouse clicks on the main panel
	 * Superclass:	java.awt.event.MouseAdapter
	 **************************************************************************************************/
	private class Mouse_Handler extends MouseAdapter
	{
		public void mouseReleased(MouseEvent e)
		{
			int xCoord = Misc.calc_x_zone(e.getX());
			int yCoord = Misc.calc_y_zone(e.getY());
			
			if (e.getButton() == e.BUTTON1)		//Show path
			{
				if (game_running == false)		//Start the game if it is not running already
				{
					game_running = true;
				}
				switch(field.calculateH(xCoord, yCoord))	//set endpoint.
				{
				case 0:
					//Continue playing
					if(!field.aStar(xCoord, yCoord))
  					field.mark_path(); //Show path
          else
             System.out.println("No path found");
				  //field.get_closed_list();
					panel.removeMouseListener(panel.getMouseListeners()[0]); //remove mouseaction
					break;
				case 1:
					break;
				}
			}
			if(e.getButton() == e.BUTTON3) //show all paths
			{
				switch(field.calculateH(xCoord, yCoord))	//set endpoint.
				{
				case 0:
					//Continue playing
					if(!field.aStar(xCoord, yCoord)) {
					 field.find_pos(); //show all paths
					 field.mark_path();
					}
					else
					 System.out.println("No path found");
					 //field.get_closed_list();
					 panel.removeMouseListener(panel.getMouseListeners()[0]); //remove mouseaction
					break;
				case 1:
					break;
				}				
			}
			if(e.getButton() == e.BUTTON2) {
				field.setWall(xCoord, yCoord);
			}
		}
	}
	
	/**************************************************************************************************
	 * Classname:	Button_Handler
	 * Properties:	Handles buttonclicks (on the button Reset only)
	 * Superclass:	- (Implements interface ActionListener)
	 **************************************************************************************************/
	private class Button_Handler implements ActionListener
	{
		public void actionPerformed(ActionEvent e)
		{
			if (e.getSource() == reset)
			{
				//Reset game
				reset_game(field.get_x_size(), field.get_y_size(), field.get_walls());
			}
		}
	}
	
	/**************************************************************************************************
	 *				
	 * Classname:	Menu_Handler 
	 * Properties:	Handles events generated by the menu of the frame
	 * Superclass:	- (Implements interface ActionListener)
	 *				
	 **************************************************************************************************/
	private class Menu_Handler implements ActionListener
	{
		public void actionPerformed(ActionEvent e)
		{
			if (e.getSource() == file_new_game)
				reset_game(field.get_x_size(), field.get_y_size(), field.get_walls());	//Reset the game
			if (e.getSource() == file_exit)
				System.exit(0);			//Quit the application
			if (e.getSource() == help_about)
				//Show about-popup
				JOptionPane.showMessageDialog(panel, "Pathfinder:\nPeter Asplund\nHenrik Hedman", "About us", JOptionPane.PLAIN_MESSAGE, new ImageIcon("sys_icon.gif"));
		}
	}
}
